export declare type Task<C = unknown, R = unknown> = (context: C) => R | Promise<R>;
export interface FixedIntervalRunOptions {
    type: 'FIXED_INTERVAL';
    /**
     * Number of milliseconds between two run start times.
     */
    interval: number;
    /**
     * Handling way when this run finished at a time which missed the next run start
     * time. Default is `RUN_IMMEDIATELY`. Notice that if multiple next run start times
     * are missed, the next run will only run once. In other words, only 1 maximum pending
     * run can exist.
     */
    onPastTime?: 'RUN_IMMEDIATELY' | 'NEXT_RUN_TIME';
}
export interface OnCompleteRunOptions {
    type: 'RUN_END_TIME';
    /**
     * Number of milliseconds to wait for the next run to start, since the last run end time.
     */
    delay: number;
}
export declare type OnSuccessNextRunOptions = FixedIntervalRunOptions | OnCompleteRunOptions;
export interface SuccessRunMetadata {
    /**
     * Start time of this attempt.
     */
    startTime: Date;
    /**
     * End time of this attempt.
     */
    endTime: Date;
}
export declare type OnSuccessNextRunEvaluator<C = unknown, R = unknown> = (returnValue: R, meta: SuccessRunMetadata, context: C) => number | Date | null;
export interface OnErrorNextRunOptions {
    delay: number;
    /**
     * Maximum number of attempt. Default is `Infinity`.
     */
    attempt?: number;
}
export interface ErrorRunMetadata {
    /**
      * The attempt number of this attempt. `1` for first attempt. `2` for second retry attempt.
      */
    attemptNumber: number;
    /**
       * Start time of this attempt.
       */
    startTime: Date;
    /**
       * End time of this attempt.
       */
    endTime: Date;
    /**
     * Start time of the first attempt.
     */
    firstAttemptStartTime: Date;
    /**
      * End time of the first attempt.
      */
    firstAttemptEndTime: Date;
}
export declare type OnErrorNextRunEvaluator<C = unknown> = (caughtValue: any, meta: ErrorRunMetadata, context: C) => number | Date | null;
export interface SingleInstanceTaskSchedulerOptions<C, R> {
    /**
     * Defautl is `null`, which will not trigger any next run.
     */
    onSuccess?: null | OnSuccessNextRunOptions | OnSuccessNextRunEvaluator<C, R>;
    /**
     * Default is `null`, which will not trigger any next run.
     */
    onError?: null | OnErrorNextRunOptions | OnErrorNextRunEvaluator<C>;
}
/**
 * A task scheduler which have at most 1 running task at any time.
 *
 * Stability: 2 - Stable.
 */
export declare class SingleInstanceTaskScheduler<C = unknown, R = unknown> {
    #private;
    constructor(task: Task<C, R>, options?: SingleInstanceTaskSchedulerOptions<C, R>);
    constructor(task: Task<C, R>, options: SingleInstanceTaskSchedulerOptions<C, R> | undefined, initialContext: C);
    get successNextRunOptions(): null | OnSuccessNextRunOptions | OnSuccessNextRunEvaluator<C, R>;
    set successNextRunOptions(value: null | OnSuccessNextRunOptions | OnSuccessNextRunEvaluator<C, R>);
    get errorNextRunOptions(): null | OnErrorNextRunOptions | OnErrorNextRunEvaluator<C>;
    set errorNextRunOptions(value: null | OnErrorNextRunOptions | OnErrorNextRunEvaluator<C>);
    /**
     * Whether a next run is scheduled.
     */
    get scheduled(): boolean;
    /**
     * The next run time of the task. `null` indicates that there will be no next run.
     */
    get nextRunTime(): null | Date;
    /**
     * Whether the task is currently running.
     */
    get running(): boolean;
    /**
     * Schedule the task to run after a given milliseconds or absolute date time. If
     * the task is already scheduled, it will be re-scheduled with attempt number retaining
     * the previous value.
     *
     * @param startDelayOrTime Start time of the next run. A delay in milliseconds, or
     * an absolute date time.
     */
    schedule(startDelayOrTime: number | Date): void;
    /**
     * Cancel the next scheduled run. Task already running will not be cancelled.
     */
    cancelNextRun(): void;
    /**
     * Run task immediately without waiting for the task returned value. Previous attempt number
     * is retained. If configured, next run will be scheduled after run completed.
     */
    run(): Promise<R>;
}
